@@ -1,6 +1,9 @@
 /* GIO - GLib Input, Output and Streaming Library
  * 
  * Copyright (C) 2006-2007 Red Hat, Inc.
+ * Copyright (C) 2022 Canonical Ltd.
+ *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -16,6 +19,7 @@
  * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
  *
  * Author: Alexander Larsson <alexl@redhat.com>
+ * Author: Marco Trevisan <marco.trevisan@canonical.com>
  */
 
 #include "config.h"
@@ -26,14 +30,6 @@
 #include <winsock2.h>
 #endif
 
-/**
- * SECTION:gioerror
- * @short_description: Error helper functions
- * @include: gio/gio.h
- *
- * Contains helper functions for reporting errors to the user.
- **/
-
 /**
  * g_io_error_quark:
  *
@@ -47,15 +43,18 @@ G_DEFINE_QUARK (g-io-error-quark, g_io_error)
  * g_io_error_from_errno:
  * @err_no: Error number as defined in errno.h.
  *
- * Converts errno.h error codes into GIO error codes. The fallback
- * value %G_IO_ERROR_FAILED is returned for error codes not currently
- * handled (but note that future GLib releases may return a more
+ * Converts `errno.h` error codes into GIO error codes.
+ *
+ * The fallback value %G_IO_ERROR_FAILED is returned for error codes not
+ * currently handled (but note that future GLib releases may return a more
  * specific value instead).
  *
- * As %errno is global and may be modified by intermediate function
- * calls, you should save its value as soon as the call which sets it
- * returns:
- * |[
+ * As `errno` is global and may be modified by intermediate function
+ * calls, you should save its value immediately after the call returns,
+ * and use the saved value instead of `errno`:
+ * 
+ *
+ * |[<!-- language="C" -->
  *   int saved_errno;
  *
  *   ret = read (blah);
@@ -64,94 +63,43 @@ G_DEFINE_QUARK (g-io-error-quark, g_io_error)
  *   g_io_error_from_errno (saved_errno);
  * ]|
  *
- * Returns: #GIOErrorEnum value for the given errno.h error number.
- **/
+ * Returns: #GIOErrorEnum value for the given `errno.h` error number
+ */
 GIOErrorEnum
 g_io_error_from_errno (gint err_no)
 {
+  GFileError file_error;
+  GIOErrorEnum io_error;
+
+  file_error = g_file_error_from_errno (err_no);
+  io_error = g_io_error_from_file_error (file_error);
+
+  if (io_error != G_IO_ERROR_FAILED)
+    return io_error;
+
   switch (err_no)
     {
-#ifdef EEXIST
-    case EEXIST:
-      return G_IO_ERROR_EXISTS;
-      break;
-#endif
-
-#ifdef EISDIR
-    case EISDIR:
-      return G_IO_ERROR_IS_DIRECTORY;
-      break;
-#endif
-
-#ifdef EACCES
-    case EACCES:
-      return G_IO_ERROR_PERMISSION_DENIED;
-      break;
-#endif
-
-#ifdef ENAMETOOLONG
-    case ENAMETOOLONG:
-      return G_IO_ERROR_FILENAME_TOO_LONG;
-      break;
-#endif
-
-#ifdef ENOENT
-    case ENOENT:
-      return G_IO_ERROR_NOT_FOUND;
-      break;
-#endif
-
-#ifdef ENOTDIR
-    case ENOTDIR:
-      return G_IO_ERROR_NOT_DIRECTORY;
-      break;
-#endif
-
-#ifdef ENXIO
-    case ENXIO:
-      return G_IO_ERROR_NOT_REGULAR_FILE;
-      break;
-#endif
-
-#ifdef EROFS
-    case EROFS:
-      return G_IO_ERROR_READ_ONLY;
-      break;
-#endif
-
 #ifdef EMLINK
     case EMLINK:
       return G_IO_ERROR_TOO_MANY_LINKS;
       break;
 #endif
 
-#ifdef ELOOP
-    case ELOOP:
-      return G_IO_ERROR_TOO_MANY_LINKS;
+#ifdef ENOMSG
+    case ENOMSG:
+      return G_IO_ERROR_INVALID_DATA;
       break;
 #endif
 
-#ifdef ENOSPC
-    case ENOSPC:
-      return G_IO_ERROR_NO_SPACE;
+#ifdef ENODATA
+    case ENODATA:
+      return G_IO_ERROR_INVALID_DATA;
       break;
 #endif
 
-#ifdef ENOMEM
-    case ENOMEM:
-      return G_IO_ERROR_NO_SPACE;
-      break;
-#endif
-      
-#ifdef EINVAL
-    case EINVAL:
-      return G_IO_ERROR_INVALID_ARGUMENT;
-      break;
-#endif
-
-#ifdef EPERM
-    case EPERM:
-      return G_IO_ERROR_PERMISSION_DENIED;
+#ifdef EBADMSG
+    case EBADMSG:
+      return G_IO_ERROR_INVALID_DATA;
       break;
 #endif
 
@@ -217,12 +165,6 @@ g_io_error_from_errno (gint err_no)
       break;
 #endif
 
-#ifdef ETXTBSY
-    case ETXTBSY:
-      return G_IO_ERROR_BUSY;
-      break;
-#endif
-
 #ifdef EWOULDBLOCK
     case EWOULDBLOCK:
       return G_IO_ERROR_WOULD_BLOCK;
@@ -236,12 +178,6 @@ g_io_error_from_errno (gint err_no)
       break;
 #endif
 
-#ifdef EMFILE
-    case EMFILE:
-      return G_IO_ERROR_TOO_MANY_OPEN_FILES;
-      break;
-#endif
-
 #ifdef EADDRINUSE
     case EADDRINUSE:
       return G_IO_ERROR_ADDRESS_IN_USE;
@@ -260,15 +196,15 @@ g_io_error_from_errno (gint err_no)
       break;
 #endif
 
-#ifdef ECONNREFUSED
-    case ECONNREFUSED:
-      return G_IO_ERROR_CONNECTION_REFUSED;
+#ifdef ENETDOWN
+    case ENETDOWN:
+      return G_IO_ERROR_NETWORK_UNREACHABLE;
       break;
 #endif
 
-#ifdef EPIPE
-    case EPIPE:
-      return G_IO_ERROR_BROKEN_PIPE;
+#ifdef ECONNREFUSED
+    case ECONNREFUSED:
+      return G_IO_ERROR_CONNECTION_REFUSED;
       break;
 #endif
 
@@ -284,6 +220,12 @@ g_io_error_from_errno (gint err_no)
       break;
 #endif
 
+#ifdef EDESTADDRREQ
+    case EDESTADDRREQ:
+      return G_IO_ERROR_DESTINATION_UNSET;
+      break;
+#endif
+
 #ifdef EMSGSIZE
     case EMSGSIZE:
       return G_IO_ERROR_MESSAGE_TOO_LARGE;
@@ -302,6 +244,70 @@ g_io_error_from_errno (gint err_no)
     }
 }
 
+/**
+ * g_io_error_from_file_error:
+ * @file_error: a #GFileError.
+ *
+ * Converts #GFileError error codes into GIO error codes.
+ *
+ * Returns: #GIOErrorEnum value for the given #GFileError error value.
+ *
+ * Since: 2.74
+ **/
+GIOErrorEnum
+g_io_error_from_file_error (GFileError file_error)
+{
+  switch (file_error)
+  {
+    case G_FILE_ERROR_EXIST:
+      return G_IO_ERROR_EXISTS;
+    case G_FILE_ERROR_ISDIR:
+      return G_IO_ERROR_IS_DIRECTORY;
+    case G_FILE_ERROR_ACCES:
+      return G_IO_ERROR_PERMISSION_DENIED;
+    case G_FILE_ERROR_NAMETOOLONG:
+      return G_IO_ERROR_FILENAME_TOO_LONG;
+    case G_FILE_ERROR_NOENT:
+      return G_IO_ERROR_NOT_FOUND;
+    case G_FILE_ERROR_NOTDIR:
+      return G_IO_ERROR_NOT_DIRECTORY;
+    case G_FILE_ERROR_NXIO:
+      return G_IO_ERROR_NOT_REGULAR_FILE;
+    case G_FILE_ERROR_NODEV:
+      return G_IO_ERROR_NO_SUCH_DEVICE;
+    case G_FILE_ERROR_ROFS:
+      return G_IO_ERROR_READ_ONLY;
+    case G_FILE_ERROR_TXTBSY:
+      return G_IO_ERROR_BUSY;
+    case G_FILE_ERROR_LOOP:
+      return G_IO_ERROR_TOO_MANY_LINKS;
+    case G_FILE_ERROR_NOSPC:
+    case G_FILE_ERROR_NOMEM:
+      return G_IO_ERROR_NO_SPACE;
+    case G_FILE_ERROR_MFILE:
+    case G_FILE_ERROR_NFILE:
+      return G_IO_ERROR_TOO_MANY_OPEN_FILES;
+    case G_FILE_ERROR_INVAL:
+      return G_IO_ERROR_INVALID_ARGUMENT;
+    case G_FILE_ERROR_PIPE:
+      return G_IO_ERROR_BROKEN_PIPE;
+    case G_FILE_ERROR_AGAIN:
+      return G_IO_ERROR_WOULD_BLOCK;
+    case G_FILE_ERROR_PERM:
+      return G_IO_ERROR_PERMISSION_DENIED;
+    case G_FILE_ERROR_NOSYS:
+      return G_IO_ERROR_NOT_SUPPORTED;
+    case G_FILE_ERROR_BADF:
+    case G_FILE_ERROR_FAILED:
+    case G_FILE_ERROR_FAULT:
+    case G_FILE_ERROR_INTR:
+    case G_FILE_ERROR_IO:
+      return G_IO_ERROR_FAILED;
+    default:
+      g_return_val_if_reached (G_IO_ERROR_FAILED);
+  }
+}
+
 #ifdef G_OS_WIN32
 
 /**
