@@ -4,6 +4,8 @@
  * 
  * Copyright (C) 2006-2007 Red Hat, Inc.
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -44,6 +46,15 @@
 #include <gstdio.h>
 #include <dirent.h>
 
+#if defined(__BIONIC__) && (__ANDROID_API__ < 26)
+#include <mntent.h>
+/* the shared object of recent bionic libc's have hasmntopt symbol, but
+   some a possible common build environment for android, termux ends
+   up with inssuficient __ANDROID_API__ value for building.
+*/
+extern char* hasmntopt(const struct mntent* mnt, const char* opt);
+#endif
+
 #if HAVE_SYS_STATFS_H
 #include <sys/statfs.h>
 #endif
@@ -76,18 +87,6 @@
 static const char *_resolve_dev_root (void);
 #endif
 
-/**
- * SECTION:gunixmounts
- * @include: gio/gunixmounts.h
- * @short_description: UNIX mounts
- *
- * Routines for managing mounted UNIX mount points and paths.
- *
- * Note that `<gio/gunixmounts.h>` belongs to the UNIX-specific GIO
- * interfaces, thus you have to use the `gio-unix-2.0.pc` pkg-config
- * file when using it.
- */
-
 /**
  * GUnixMountType:
  * @G_UNIX_MOUNT_TYPE_UNKNOWN: Unknown UNIX mount type.
@@ -202,6 +201,11 @@ static GSource *proc_mounts_watch_source;
 #define endmntent(f) fclose(f)
 #endif
 
+#ifdef HAVE_LIBMOUNT
+/* Protected by proc_mounts_source lock */
+static struct libmnt_monitor *proc_mounts_monitor = NULL;
+#endif
+
 static gboolean
 is_in (const char *value, const char *set[])
 {
@@ -264,10 +268,12 @@ g_unix_is_mount_path_system_internal (const char *mount_path)
     "/var",
     "/var/crash",
     "/var/local",
+    GLIB_LOCALSTATEDIR,
     "/var/log",
     "/var/log/audit", /* https://bugzilla.redhat.com/show_bug.cgi?id=333041 */
     "/var/mail",
     "/var/run",
+    GLIB_RUNSTATEDIR,
     "/var/tmp",       /* https://bugzilla.redhat.com/show_bug.cgi?id=335241 */
     "/proc",
     "/sbin",
@@ -581,7 +587,7 @@ _g_get_unix_mounts (void)
   
   read_file = get_mtab_read_file ();
 
-  file = setmntent (read_file, "r");
+  file = setmntent (read_file, "re");
   if (file == NULL)
     return NULL;
 
@@ -723,7 +729,7 @@ _g_get_unix_mounts (void)
   
   read_file = get_mtab_read_file ();
   
-  file = setmntent (read_file, "r");
+  file = setmntent (read_file, "re");
   if (file == NULL)
     return NULL;
   
@@ -1110,7 +1116,7 @@ _g_get_unix_mount_points (void)
   
   read_file = get_fstab_file ();
   
-  file = setmntent (read_file, "r");
+  file = setmntent (read_file, "re");
   if (file == NULL)
     return NULL;
 
@@ -1199,7 +1205,7 @@ _g_get_unix_mount_points (void)
   
   read_file = get_fstab_file ();
   
-  file = setmntent (read_file, "r");
+  file = setmntent (read_file, "re");
   if (file == NULL)
     return NULL;
 
@@ -1374,7 +1380,7 @@ _g_get_unix_mount_points (void)
   
   read_file = get_fstab_file ();
   
-  file = setmntent (read_file, "r");
+  file = setmntent (read_file, "re");
   if (file == NULL)
     return NULL;
   
@@ -1411,6 +1417,7 @@ _g_get_unix_mount_points (void)
   GList *return_list = NULL;
   G_LOCK_DEFINE_STATIC (fsent);
 #ifdef HAVE_SYS_SYSCTL_H
+  uid_t uid = getuid ();
   int usermnt = 0;
   struct stat sb;
 #endif
@@ -1463,14 +1470,13 @@ _g_get_unix_mount_points (void)
 
 #ifdef HAVE_SYS_SYSCTL_H
       if (usermnt != 0)
-	{
-	  uid_t uid = getuid ();
-	  if (stat (fstab->fs_file, &sb) == 0)
-	    {
-	      if (uid == 0 || sb.st_uid == uid)
-		is_user_mountable = TRUE;
-	    }
-	}
+        {
+          if (uid == 0 ||
+              (stat (fstab->fs_file, &sb) == 0 && sb.st_uid == uid))
+            {
+              is_user_mountable = TRUE;
+            }
+        }
 #endif
 
       mount_point = create_unix_mount_point (fstab->fs_spec,
@@ -1892,7 +1898,36 @@ proc_mounts_changed (GIOChannel   *channel,
                      GIOCondition  cond,
                      gpointer      user_data)
 {
+  gboolean has_changed = FALSE;
+
+#ifdef HAVE_LIBMOUNT
+  if (cond & G_IO_IN)
+    {
+      G_LOCK (proc_mounts_source);
+      if (proc_mounts_monitor != NULL)
+        {
+          int ret;
+
+          /* The mnt_monitor_next_change function needs to be used to avoid false-positives. */
+          ret = mnt_monitor_next_change (proc_mounts_monitor, NULL, NULL);
+          if (ret == 0)
+            {
+              has_changed = TRUE;
+              ret = mnt_monitor_event_cleanup (proc_mounts_monitor);
+            }
+
+          if (ret < 0)
+            g_debug ("mnt_monitor_next_change failed: %s", g_strerror (-ret));
+        }
+      G_UNLOCK (proc_mounts_source);
+    }
+
+#else
   if (cond & G_IO_ERR)
+    has_changed = TRUE;
+#endif
+
+  if (has_changed)
     {
       G_LOCK (proc_mounts_source);
       mount_poller_time = (guint64) g_get_monotonic_time ();
@@ -1957,6 +1992,10 @@ mount_monitor_stop (void)
       g_source_destroy (proc_mounts_watch_source);
       proc_mounts_watch_source = NULL;
     }
+
+#ifdef HAVE_LIBMOUNT
+  g_clear_pointer (&proc_mounts_monitor, mnt_unref_monitor);
+#endif
   G_UNLOCK (proc_mounts_source);
 
   if (mtab_monitor)
@@ -1998,9 +2037,43 @@ mount_monitor_start (void)
        */
       if (g_str_has_prefix (mtab_path, "/proc/"))
         {
-          GIOChannel *proc_mounts_channel;
+          GIOChannel *proc_mounts_channel = NULL;
           GError *error = NULL;
+#ifdef HAVE_LIBMOUNT
+          int ret;
+
+          G_LOCK (proc_mounts_source);
+
+          proc_mounts_monitor = mnt_new_monitor ();
+          ret = mnt_monitor_enable_kernel (proc_mounts_monitor, TRUE);
+          if (ret < 0)
+            g_warning ("mnt_monitor_enable_kernel failed: %s", g_strerror (-ret));
+
+          ret = mnt_monitor_enable_userspace (proc_mounts_monitor, TRUE, NULL);
+          if (ret < 0)
+            g_warning ("mnt_monitor_enable_userspace failed: %s", g_strerror (-ret));
+
+#ifdef HAVE_MNT_MONITOR_VEIL_KERNEL
+          ret = mnt_monitor_veil_kernel (proc_mounts_monitor, TRUE);
+          if (ret < 0)
+            g_warning ("mnt_monitor_veil_kernel failed: %s", g_strerror (-ret));
+#endif
+
+          ret = mnt_monitor_get_fd (proc_mounts_monitor);
+          if (ret >= 0)
+            {
+              proc_mounts_channel = g_io_channel_unix_new (ret);
+            }
+          else
+            {
+              g_set_error_literal (&error, G_IO_ERROR, g_io_error_from_errno (-ret),
+                                   g_strerror (-ret));
+            }
+
+          G_UNLOCK (proc_mounts_source);
+#else
           proc_mounts_channel = g_io_channel_new_file (mtab_path, "r", &error);
+#endif
           if (proc_mounts_channel == NULL)
             {
               g_warning ("Error creating IO channel for %s: %s (%s, %d)", mtab_path,
@@ -2011,7 +2084,11 @@ mount_monitor_start (void)
             {
               G_LOCK (proc_mounts_source);
 
+#ifdef HAVE_LIBMOUNT
+              proc_mounts_watch_source = g_io_create_watch (proc_mounts_channel, G_IO_IN);
+#else
               proc_mounts_watch_source = g_io_create_watch (proc_mounts_channel, G_IO_ERR);
+#endif
               mount_poller_time = (guint64) g_get_monotonic_time ();
               g_source_set_callback (proc_mounts_watch_source,
                                      (GSourceFunc) proc_mounts_changed,
@@ -3032,7 +3109,7 @@ g_unix_mount_point_guess_can_eject (GUnixMountPoint *mount_point)
 /* Utility functions {{{1 */
 
 #ifdef HAVE_MNTENT_H
-/* borrowed from gtk/gtkfilesystemunix.c in GTK+ on 02/23/2006 */
+/* borrowed from gtk/gtkfilesystemunix.c in GTK on 02/23/2006 */
 static void
 _canonicalize_filename (gchar *filename)
 {
@@ -3162,7 +3239,7 @@ _resolve_dev_root (void)
           /* see if device with similar major:minor as /dev/root is mention
            * in /etc/mtab (it usually is) 
            */
-          f = fopen ("/etc/mtab", "r");
+          f = fopen ("/etc/mtab", "re");
           if (f != NULL) 
             {
 	      struct mntent *entp;
