@@ -56,105 +56,6 @@
 
 #include "glibintl.h"
 
-/**
- * SECTION:conversions
- * @title: Character Set Conversion
- * @short_description: convert strings between different character sets
- *
- * The g_convert() family of function wraps the functionality of iconv().
- * In addition to pure character set conversions, GLib has functions to
- * deal with the extra complications of encodings for file names.
- *
- * ## File Name Encodings
- *
- * Historically, UNIX has not had a defined encoding for file names:
- * a file name is valid as long as it does not have path separators
- * in it ("/"). However, displaying file names may require conversion:
- * from the character set in which they were created, to the character
- * set in which the application operates. Consider the Spanish file name
- * "Presentación.sxi". If the application which created it uses
- * ISO-8859-1 for its encoding,
- * |[
- * Character:  P  r  e  s  e  n  t  a  c  i  ó  n  .  s  x  i
- * Hex code:   50 72 65 73 65 6e 74 61 63 69 f3 6e 2e 73 78 69
- * ]|
- * However, if the application use UTF-8, the actual file name on
- * disk would look like this:
- * |[
- * Character:  P  r  e  s  e  n  t  a  c  i  ó     n  .  s  x  i
- * Hex code:   50 72 65 73 65 6e 74 61 63 69 c3 b3 6e 2e 73 78 69
- * ]|
- * GLib uses UTF-8 for its strings, and GUI toolkits like GTK that use
- * GLib do the same thing. If you get a file name from the file system,
- * for example, from readdir() or from g_dir_read_name(), and you wish
- * to display the file name to the user, you  will need to convert it
- * into UTF-8. The opposite case is when the user types the name of a
- * file they wish to save: the toolkit will give you that string in
- * UTF-8 encoding, and you will need to convert it to the character
- * set used for file names before you can create the file with open()
- * or fopen().
- *
- * By default, GLib assumes that file names on disk are in UTF-8
- * encoding. This is a valid assumption for file systems which
- * were created relatively recently: most applications use UTF-8
- * encoding for their strings, and that is also what they use for
- * the file names they create. However, older file systems may
- * still contain file names created in "older" encodings, such as
- * ISO-8859-1. In this case, for compatibility reasons, you may want
- * to instruct GLib to use that particular encoding for file names
- * rather than UTF-8. You can do this by specifying the encoding for
- * file names in the [`G_FILENAME_ENCODING`][G_FILENAME_ENCODING]
- * environment variable. For example, if your installation uses
- * ISO-8859-1 for file names, you can put this in your `~/.profile`:
- * |[
- * export G_FILENAME_ENCODING=ISO-8859-1
- * ]|
- * GLib provides the functions g_filename_to_utf8() and
- * g_filename_from_utf8() to perform the necessary conversions.
- * These functions convert file names from the encoding specified
- * in `G_FILENAME_ENCODING` to UTF-8 and vice-versa. This
- * [diagram][file-name-encodings-diagram] illustrates how
- * these functions are used to convert between UTF-8 and the
- * encoding for file names in the file system.
- *
- * ## Conversion between file name encodings # {#file-name-encodings-diagram)
- *
- * ![](file-name-encodings.png)
- *
- * ## Checklist for Application Writers
- *
- * This section is a practical summary of the detailed
- * things to do to make sure your applications process file
- * name encodings correctly.
- * 
- * 1. If you get a file name from the file system from a function
- *    such as readdir() or gtk_file_chooser_get_filename(), you do
- *    not need to do any conversion to pass that file name to
- *    functions like open(), rename(), or fopen() -- those are "raw"
- *    file names which the file system understands.
- *
- * 2. If you need to display a file name, convert it to UTF-8 first
- *    by using g_filename_to_utf8(). If conversion fails, display a
- *    string like "Unknown file name". Do not convert this string back
- *    into the encoding used for file names if you wish to pass it to
- *    the file system; use the original file name instead.
- *
- *    For example, the document window of a word processor could display
- *    "Unknown file name" in its title bar but still let the user save
- *    the file, as it would keep the raw file name internally. This
- *    can happen if the user has not set the `G_FILENAME_ENCODING`
- *    environment variable even though they have files whose names are
- *    not encoded in UTF-8.
- *
- * 3. If your user interface lets the user type a file name for saving
- *    or renaming, convert it to the encoding used for file names in
- *    the file system by using g_filename_from_utf8(). Pass the converted
- *    file name to functions like fopen(). If conversion fails, ask the
- *    user to enter a different file name. This can happen if the user
- *    types Japanese characters when `G_FILENAME_ENCODING` is set to
- *    `ISO-8859-1`, for example.
- */
-
 /* We try to terminate strings in unknown charsets with this many zero bytes
  * to ensure that multibyte strings really are nul-terminated when we return
  * them from g_convert() and friends.
@@ -172,8 +73,17 @@ try_conversion (const char *to_codeset,
 
   if (*cd == (iconv_t)-1 && errno == EINVAL)
     return FALSE;
-  else
-    return TRUE;
+
+#if defined(__FreeBSD__) && defined(ICONV_SET_ILSEQ_INVALID)
+  /* On FreeBSD request GNU iconv compatible handling of characters that cannot
+   * be repesented in the destination character set.
+   * See https://cgit.freebsd.org/src/commit/?id=7c5b23111c5fd1992047922d4247c4a1ce1bb6c3
+   */
+  int value = 1;
+  if (iconvctl (*cd, ICONV_SET_ILSEQ_INVALID, &value) != 0)
+    return FALSE;
+#endif
+  return TRUE;
 }
 
 static gboolean
@@ -1662,7 +1572,12 @@ hostname_validate (const char *hostname)
  *         errors. Any of the errors in #GConvertError may occur.
  * 
  * Converts an escaped ASCII-encoded URI to a local filename in the
- * encoding used for filenames. 
+ * encoding used for filenames.
+ *
+ * Since GLib 2.78, the query string and fragment can be present in the URI,
+ * but are not part of the resulting filename.
+ * We take inspiration from https://url.spec.whatwg.org/#file-state,
+ * but we don't support the entire standard.
  * 
  * Returns: (type filename): a newly-allocated string holding
  *               the resulting filename, or %NULL on an error.
@@ -1672,11 +1587,13 @@ g_filename_from_uri (const gchar *uri,
 		     gchar      **hostname,
 		     GError     **error)
 {
-  const char *path_part;
+  const char *past_scheme;
   const char *host_part;
   char *unescaped_hostname;
   char *result;
   char *filename;
+  char *past_path;
+  char *temp_uri;
   int offs;
 #ifdef G_OS_WIN32
   char *p, *slash;
@@ -1692,40 +1609,44 @@ g_filename_from_uri (const gchar *uri,
 		   uri);
       return NULL;
     }
-  
-  path_part = uri + strlen ("file:");
-  
-  if (strchr (path_part, '#') != NULL)
-    {
-      g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_BAD_URI,
-		   _("The local file URI “%s” may not include a “#”"),
-		   uri);
-      return NULL;
-    }
-	
-  if (has_case_prefix (path_part, "///")) 
-    path_part += 2;
-  else if (has_case_prefix (path_part, "//"))
-    {
-      path_part += 2;
-      host_part = path_part;
 
-      path_part = strchr (path_part, '/');
+  temp_uri = g_strdup (uri);
 
-      if (path_part == NULL)
+  past_scheme = temp_uri + strlen ("file:");
+  
+  past_path = strchr (past_scheme, '?');
+  if (past_path != NULL)
+    *past_path = '\0';
+
+  past_path = strchr (past_scheme, '#');
+  if (past_path != NULL)
+    *past_path = '\0';
+
+  if (has_case_prefix (past_scheme, "///"))
+    past_scheme += 2;
+  else if (has_case_prefix (past_scheme, "//"))
+    {
+      past_scheme += 2;
+      host_part = past_scheme;
+
+      past_scheme = strchr (past_scheme, '/');
+
+      if (past_scheme == NULL)
 	{
+          g_free (temp_uri);
 	  g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_BAD_URI,
 		       _("The URI “%s” is invalid"),
 		       uri);
 	  return NULL;
 	}
 
-      unescaped_hostname = g_unescape_uri_string (host_part, path_part - host_part, "", TRUE);
+      unescaped_hostname = g_unescape_uri_string (host_part, past_scheme - host_part, "", TRUE);
 
       if (unescaped_hostname == NULL ||
 	  !hostname_validate (unescaped_hostname))
 	{
 	  g_free (unescaped_hostname);
+          g_free (temp_uri);
 	  g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_BAD_URI,
 		       _("The hostname of the URI “%s” is invalid"),
 		       uri);
@@ -1738,10 +1659,11 @@ g_filename_from_uri (const gchar *uri,
 	g_free (unescaped_hostname);
     }
 
-  filename = g_unescape_uri_string (path_part, -1, "/", FALSE);
+  filename = g_unescape_uri_string (past_scheme, -1, "/", FALSE);
 
   if (filename == NULL)
     {
+      g_free (temp_uri);
       g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_BAD_URI,
 		   _("The URI “%s” contains invalidly escaped characters"),
 		   uri);
@@ -1785,6 +1707,8 @@ g_filename_from_uri (const gchar *uri,
   result = g_strdup (filename + offs);
   g_free (filename);
 
+  g_free (temp_uri);
+
   return result;
 }
 
